@@ -23,15 +23,19 @@
 
 #include <gnome.h>
 
-#include "globals.h"
+#include "appdata.h"
 #include "diagram.h"
-#include "preferences.h"
+#include "pref_dialog.h"
 #include "node.h"
 #include "info_windows.h"
 #include "protocols.h"
 #include "datastructs.h"
+#include "ip-cache.h"
+#include "menus.h"
 #include "capture.h"
 #include "conversations.h"
+#include "preferences.h"
+#include "export.h"
 
 /* maximum node and link size */
 #define MAX_NODE_SIZE 5000
@@ -70,7 +74,8 @@ static gint canvas_node_update(node_id_t  * ether_addr,
 typedef struct
 {
   link_id_t canvas_link_id; /* id of the link */
-  GnomeCanvasItem *link_item;
+  GnomeCanvasItem *src_item;    /* triangle for src side */
+  GnomeCanvasItem *dst_item;    /* triangle for dst side */
   GdkColor color;
 }
 canvas_link_t;
@@ -132,12 +137,15 @@ static gint reposition_canvas_nodes (guint8 * ether_addr,
 static gint check_new_link (link_id_t * link_id,
 			    link_t * link, GtkWidget * canvas);
 static gdouble get_node_size (gdouble average);
-static gdouble get_link_size (gdouble average);
+static gdouble get_link_size(const basic_stats_t *link_stats);
 static gint link_item_event (GnomeCanvasItem * item,
 			     GdkEvent * event, canvas_link_t * canvas_link);
 static gint node_item_event (GnomeCanvasItem * item,
 			     GdkEvent * event, canvas_node_t * canvas_node);
 static void update_legend(void);
+static void draw_oneside_link(double xs, double ys, double xd, double yd,
+                              const basic_stats_t *link_data, 
+                              guint32 scaledColor, GnomeCanvasItem *item);
 
 
 void ask_reposition(gboolean r_font)
@@ -158,7 +166,7 @@ void dump_stats(guint32 diff_msecs)
                                    g_tree_nnodes(canvas_nodes), displayed_nodes, 
                                    links_catalog_size(), active_conversations(), 
                                    active_names(), protocol_summary_size(),
-                                   n_packets, total_mem_packets, 
+                                   appdata.n_packets, appdata.total_mem_packets, 
                                    packet_list_item_count(), ipc,
                                    canvas_obj_count,
                                    (unsigned int) diff_msecs);
@@ -204,7 +212,7 @@ init_diagram (GladeXML *xml)
   initialize_pref_controls();
   
   /* Sets canvas background to black */
-  canvas = glade_xml_get_widget (xml, "canvas1");
+  canvas = glade_xml_get_widget (appdata.xml, "canvas1");
 
   gdk_color_parse ("black", &black_color);
   gdk_colormap_alloc_color (gdk_colormap_get_system (), &black_color,
@@ -225,17 +233,6 @@ init_diagram (GladeXML *xml)
 }				/* init_diagram */
 
 
-void change_refresh_period(guint32 newperiod)
-{
-  pref.refresh_period = newperiod;
-  /* When removing the source (which could either be an idle or a timeout
-   * function, I'm also forcing the callback for the corresponding 
-   * destroying function, which in turn will install a timeout or idle
-   * function using the new refresh_period. It might take a while for it
-   * to settle down, but I think it works now */
-  g_source_remove (diagram_timeout);
-}
-
 void
 destroying_timeout (gpointer data)
 {
@@ -370,13 +367,20 @@ diagram_update_links(GtkWidget * canvas)
  * 2. Updates nodes looks
  * 3. Updates links looks
  */
-guint
-update_diagram (GtkWidget * canvas)
+guint update_diagram(GtkWidget * canvas)
 {
-  static struct timeval last_refresh_time = { 0, 0 }, diff;
-  guint32 diff_msecs;
+  static struct timeval last_refresh_time = { 0, 0 };
+  double diffms;
   enum status_t status;
 
+  /* if requested and enabled, dump to xml */
+  if (appdata.request_dump && appdata.export_file_signal)
+    {
+      g_warning (_("SIGUSR1 received: exporting to %s"), appdata.export_file_signal);
+      dump_xml(appdata.export_file_signal);
+      appdata.request_dump = FALSE; 
+    }
+  
   status = get_capture_status();
   if (status == PAUSE)
     return FALSE;
@@ -402,7 +406,7 @@ update_diagram (GtkWidget * canvas)
     }
 
   already_updating = TRUE;
-  gettimeofday (&now, NULL);
+  gettimeofday (&appdata.now, NULL);
 
   /* update nodes */
   diagram_update_nodes(canvas);
@@ -423,28 +427,27 @@ update_diagram (GtkWidget * canvas)
    * CPU with redraws */
 
   if ((last_refresh_time.tv_sec == 0) && (last_refresh_time.tv_usec == 0))
-    last_refresh_time = now;
+    last_refresh_time = appdata.now;
 
   /* Force redraw */
   while (gtk_events_pending ())
     gtk_main_iteration ();
 
-  gettimeofday (&now, NULL);
-  diff = substract_times (now, last_refresh_time);
-  diff_msecs = diff.tv_sec * 1000 + diff.tv_usec / 1000;
-  last_refresh_time = now;
+  gettimeofday (&appdata.now, NULL);
+  diffms = substract_times_ms(&appdata.now, &last_refresh_time);
+  last_refresh_time = appdata.now;
 
   already_updating = FALSE;
 
   if (!is_idle)
     {
-      if (diff_msecs > pref.refresh_period * 1.2)
-	  return FALSE;		/* Removes the timeout */
+      if (diffms > pref.refresh_period * 1.2)
+        return FALSE;		/* Removes the timeout */
     }
   else
     {
-      if (diff_msecs < pref.refresh_period)
-	  return FALSE;		/* removes the idle */
+      if (diffms < pref.refresh_period)
+        return FALSE;		/* removes the idle */
     }
 
   if (stop_requested)
@@ -473,7 +476,7 @@ update_legend()
   GtkWidget *prot_table;
 
   /* first, check if there are expired protocols */
-  prot_table = glade_xml_get_widget (xml, "prot_table");
+  prot_table = glade_xml_get_widget (appdata.xml, "prot_table");
   if (!prot_table)
     return;
 
@@ -565,7 +568,7 @@ delete_gui_protocols (void)
   protohash_reset_cycle();
 
   /* remove proto labels from legend */
-  prot_table = GTK_CONTAINER (glade_xml_get_widget (xml, "prot_table"));
+  prot_table = GTK_CONTAINER (glade_xml_get_widget (appdata.xml, "prot_table"));
   item = gtk_container_get_children (GTK_CONTAINER (prot_table));
   while (item)
     {
@@ -575,7 +578,7 @@ delete_gui_protocols (void)
 
   /* resize legend */
   gtk_container_resize_children(prot_table);
-  gtk_widget_queue_resize (GTK_WIDGET (app1));
+  gtk_widget_queue_resize (GTK_WIDGET (appdata.app1));
 }				/* delete_gui_protocols */
 
 /* Checks if there is a canvas_node per each node. If not, one canvas_node
@@ -693,6 +696,9 @@ canvas_node_update(node_id_t * node_id, canvas_node_t * canvas_node,
     case INST_OUTBOUND:
       node_size = get_node_size (node->node_stats.stats_out.average);
       break;
+    case INST_PACKETS:
+      node_size = get_node_size (node->node_stats.pkt_list.length);
+      break;
     case ACCU_TOTAL:
       node_size = get_node_size (node->node_stats.stats.accumulated);
       break;
@@ -702,6 +708,12 @@ canvas_node_update(node_id_t * node_id, canvas_node_t * canvas_node,
     case ACCU_OUTBOUND:
       node_size = get_node_size (node->node_stats.stats_out.accumulated);
       break;
+    case ACCU_PACKETS:
+      node_size = get_node_size (node->node_stats.stats.accu_packets);
+      break;
+    case ACCU_AVG_SIZE:
+      node_size = get_node_size (node->node_stats.stats.avg_size);
+      break;
     default:
       node_size = get_node_size (node->node_stats.stats_out.average);
       g_warning (_("Unknown value or node_size_variable"));
@@ -771,19 +783,20 @@ canvas_node_update(node_id_t * node_id, canvas_node_t * canvas_node,
 static gboolean
 display_node (node_t * node)
 {
-  struct timeval diff;
+  double diffms;
 
   if (!node)
     return FALSE;
 
-  diff = substract_times (now, node->node_stats.stats.last_time);
+  diffms = substract_times_ms(&appdata.now, &node->node_stats.stats.last_time);
 
   /* There are problems if a canvas_node is deleted if it still
    * has packets, so we have to check that as well */
 
   /* Remove canvas_node if node is too old */
-  if (IS_OLDER (diff, pref.gui_node_timeout_time)
-      && pref.gui_node_timeout_time && !node->node_stats.pkt_list.length)
+  if (diffms >= pref.gui_node_timeout_time
+      && pref.gui_node_timeout_time 
+      && !node->node_stats.pkt_list.length)
     return FALSE;
 
 #if 1
@@ -804,13 +817,13 @@ limit_nodes (void)
   displayed_nodes = 0;		/* We'll increment for each node we don't
 				 * limit */
 
-  if (pref.node_limit < 0)
+  if (appdata.node_limit < 0)
     {
       displayed_nodes = g_tree_nnodes (canvas_nodes);
       return;
     }
 
-  limit = pref.node_limit;
+  limit = appdata.node_limit;
 
   ordered_nodes = g_tree_new (traffic_compare);
 
@@ -1027,20 +1040,28 @@ check_new_link (link_id_t * link_id, link_t * link, GtkWidget * canvas)
       for (; i <= 5; i++)
 	points->coords[i] = 0.0;
 
-      new_canvas_link->link_item
+      new_canvas_link->src_item
 	= gnome_canvas_item_new (group,
 				 gnome_canvas_polygon_get_type (),
 				 "points", points, "fill_color", "tan", NULL);
-      addref_canvas_obj(G_OBJECT (new_canvas_link->link_item));
+      addref_canvas_obj(G_OBJECT (new_canvas_link->src_item));
 
+      new_canvas_link->dst_item
+	= gnome_canvas_item_new (group,
+				 gnome_canvas_polygon_get_type (),
+				 "points", points, "fill_color", "tan", NULL);
+      addref_canvas_obj(G_OBJECT (new_canvas_link->dst_item));
 
       g_tree_insert (canvas_links,
 		     &new_canvas_link->canvas_link_id, new_canvas_link);
-      gnome_canvas_item_lower_to_bottom (new_canvas_link->link_item);
+      gnome_canvas_item_lower_to_bottom (new_canvas_link->src_item);
+      gnome_canvas_item_lower_to_bottom (new_canvas_link->dst_item);
 
       gnome_canvas_points_unref (points);
 
-      g_signal_connect (G_OBJECT (new_canvas_link->link_item), "event",
+      g_signal_connect (G_OBJECT (new_canvas_link->src_item), "event",
+			(GtkSignalFunc) link_item_event, new_canvas_link);
+      g_signal_connect (G_OBJECT (new_canvas_link->dst_item), "event",
 			(GtkSignalFunc) link_item_event, new_canvas_link);
 
     }
@@ -1056,17 +1077,15 @@ static gint
 canvas_link_update(link_id_t * link_id, canvas_link_t * canvas_link,
 		     GList **delete_list)
 {
-  link_t *link;
-  GnomeCanvasPoints *points;
-  canvas_node_t *canvas_node;
-  gdouble link_size, versorx, versory, modulus;
+  const link_t *link;
+  const canvas_node_t *canvas_dst;
+  const canvas_node_t *canvas_src;
   guint32 scaledColor;
-  gdouble scale;
-  double dx, dy;		/* temporary */
+  double xs, ys, xd, yd, scale;
 
-/* We used to run update_link here, but that was a major performance penalty, and now it is done in update_diagram */
+  /* We used to run update_link here, but that was a major performance penalty, 
+   * and now it is done in update_diagram */
   link = links_catalog_find(link_id);
-
   if (!link)
     {
       *delete_list = g_list_prepend( *delete_list, link_id);
@@ -1074,63 +1093,37 @@ canvas_link_update(link_id_t * link_id, canvas_link_t * canvas_link,
       return FALSE;
     }
 
-
-  points = gnome_canvas_points_new (3);
-
   /* If either source or destination has disappeared, we hide the link
    * until it can be show again */
-  /* TODO: This is a dirty hack. Redo this again later by properly 
-   * deleting the link */
 
   /* We get coords for the destination node */
-  canvas_node = g_tree_lookup (canvas_nodes, &link_id->dst);
-  if (!canvas_node || !canvas_node->shown)
+  canvas_dst = g_tree_lookup (canvas_nodes, &link_id->dst);
+  if (!canvas_dst || !canvas_dst->shown)
     {
-      gnome_canvas_item_hide (canvas_link->link_item);
-      gnome_canvas_points_unref (points);
+      gnome_canvas_item_hide (canvas_link->src_item);
+      gnome_canvas_item_hide (canvas_link->dst_item);
       return FALSE;
     }
-  g_object_get (G_OBJECT (canvas_node->group_item), "x", &points->coords[0],
-		"y", &points->coords[1], NULL);
 
   /* We get coords from source node */
-  canvas_node = g_tree_lookup (canvas_nodes, &link_id->src);
-  if (!canvas_node || !canvas_node->shown)
+  canvas_src = g_tree_lookup (canvas_nodes, &link_id->src);
+  if (!canvas_src || !canvas_src->shown)
     {
-      gnome_canvas_item_hide (canvas_link->link_item);
-      gnome_canvas_points_unref (points);
+      gnome_canvas_item_hide (canvas_link->src_item);
+      gnome_canvas_item_hide (canvas_link->dst_item);
       return FALSE;
     }
 
-  g_object_get (G_OBJECT (canvas_node->group_item), "x", &dx, "y", &dy, NULL);
-  versorx = -(points->coords[1] - dy);
-  versory = points->coords[0] - dx;
-
-  modulus = sqrt (pow (versorx, 2) + pow (versory, 2));
-  link_size = get_link_size (link->link_stats.stats.average) / 2;
-
-  /* limit the maximum size to avoid overload */
-  if (link_size > MAX_LINK_SIZE)
-    link_size = MAX_LINK_SIZE; 
-
-  points->coords[2] = dx + (versorx / modulus) * link_size;
-  points->coords[3] = dy + (versory / modulus) * link_size;
-  points->coords[4] = dx - (versorx / modulus) * link_size;
-  points->coords[5] = dy - (versory / modulus) * link_size;
-
-  /* TODO What if there never is a protocol?
+  /* What if there never is a protocol?
    * I have to initialize canvas_link->color to a known value */
   if (link->main_prot[pref.stack_level])
     {
+      double diffms;
       canvas_link->color = protohash_color(link->main_prot[pref.stack_level]);
 
       /* scale color down to 10% at link timeout */
-      struct timeval diff;
-      diff = substract_times (now, link->link_stats.stats.last_time);
-      scale =
-        pow (0.10,
-             (diff.tv_sec * 1000.0 +
-              diff.tv_usec / 1000) / pref.gui_link_timeout_time);
+      diffms = substract_times_ms(&appdata.now, &link->link_stats.stats.last_time);
+      scale = pow(0.10, diffms / pref.gui_link_timeout_time);
 
       scaledColor =
         (((int) (scale * canvas_link->color.red) & 0xFF00) << 16) |
@@ -1143,19 +1136,64 @@ canvas_link_update(link_id_t * link_id, canvas_link_t * canvas_link,
       scaledColor = black;
     }
 
-  gnome_canvas_item_set (canvas_link->link_item, 
-                          "points", points,
-                          "fill_color_rgba", scaledColor, NULL);
+  /* retrieve coordinates of node centers */
+  g_object_get (G_OBJECT (canvas_src->group_item), "x", &xs, "y", &ys, NULL);
+  g_object_get (G_OBJECT (canvas_dst->group_item), "x", &xd, "y", &yd, NULL);
 
-  /* If we got this far, the link can be shown. Make sure it is */
-  gnome_canvas_item_show (canvas_link->link_item);
+  /* first draw triangle for src->dst */
+  draw_oneside_link(xs, ys, xd, yd, &(link->link_stats.stats_out), scaledColor, 
+                    canvas_link->src_item);
 
-  gnome_canvas_points_unref (points);
+  /* then draw triangle for dst->src */
+  draw_oneside_link(xd, yd, xs, ys, &(link->link_stats.stats_in), scaledColor, 
+                    canvas_link->dst_item);
 
   return FALSE;
 
 }				/* update_canvas_links */
 
+/* given the src and dst node centers, plus a size, draws a triangle in the 
+ * specified color on the provided canvas item*/
+static void draw_oneside_link(double xs, double ys, double xd, double yd,
+                              const basic_stats_t *link_stats, 
+                              guint32 scaledColor, GnomeCanvasItem *item)
+{
+  GnomeCanvasPoints *points;
+  gdouble versorx, versory, modulus, link_size;
+
+  link_size = get_link_size(link_stats) / 2;
+
+  /* limit the maximum size to avoid overload */
+  if (link_size > MAX_LINK_SIZE)
+    link_size = MAX_LINK_SIZE; 
+
+  versorx = -(yd - ys);
+  versory = xd - xs;
+  modulus = sqrt (pow (versorx, 2) + pow (versory, 2));
+  if (modulus == 0)
+    {
+      link_size = 0;
+      modulus = 1;
+    }
+
+  points = gnome_canvas_points_new (3);
+  points->coords[0] = xd;
+  points->coords[1] = yd;
+  points->coords[2] = xs + versorx * link_size / modulus;
+  points->coords[3] = ys + versory * link_size / modulus;
+  points->coords[4] = xs - versorx * link_size / modulus;
+  points->coords[5] = ys - versory * link_size / modulus;
+
+  gnome_canvas_item_set (item, 
+                          "points", points,
+                          "fill_color_rgba", scaledColor, NULL);
+
+  /* If we got this far, the link can be shown. Make sure it is */
+  gnome_canvas_item_show (item);
+  gnome_canvas_points_unref (points);
+}
+
+
 
 /* Returs the radius in pixels given average traffic and size mode */
 static gdouble
@@ -1178,20 +1216,46 @@ get_node_size (gdouble average)
 }
 
 /* Returs the width in pixels given average traffic and size mode */
-static gdouble
-get_link_size (gdouble average)
+static gdouble get_link_size (const basic_stats_t *link_stats)
 {
   gdouble result = 0.0;
+  gdouble data;
+
+  /* since links are one-sided, there's no distinction between inbound/outbound
+     data.   */
+  switch (pref.node_size_variable)
+    {
+    case INST_TOTAL:
+    case INST_INBOUND:
+    case INST_OUTBOUND:
+    case INST_PACKETS: /* active packets stat not available */
+      data = link_stats->average;
+      break;
+    case ACCU_TOTAL:
+    case ACCU_INBOUND:
+    case ACCU_OUTBOUND:
+      data = link_stats->accumulated;
+      break;
+    case ACCU_PACKETS:
+      data = link_stats->accu_packets;
+      break;
+    case ACCU_AVG_SIZE:
+      data = link_stats->avg_size;
+      break;
+    default:
+      data = link_stats->average;
+      g_warning (_("Unknown value for link_size_variable"));
+    }
   switch (pref.size_mode)
     {
     case LINEAR:
-      result = average + 1;
+      result = data + 1;
       break;
     case LOG:
-      result = log (average + 1);
+      result = log(data + 1);
       break;
     case SQRT:
-      result = sqrt (average + 1);
+      result = sqrt(data + 1);
       break;
     }
   return 1.0 + pref.node_radius_multiplier * pref.link_node_ratio * result;
@@ -1223,11 +1287,11 @@ link_item_event (GnomeCanvasItem * item, GdkEvent * event,
 			   link->main_prot[pref.stack_level]);
       else
 	str = g_strdup_printf (_("Link main protocol unknown"));
-      gtk_statusbar_push(statusbar, 1, str);
+      gtk_statusbar_push(appdata.statusbar, 1, str);
       g_free (str);
       break;
     case GDK_LEAVE_NOTIFY:
-      gtk_statusbar_pop(statusbar, 1);
+      gtk_statusbar_pop(appdata.statusbar, 1);
       break;
     default:
       break;
@@ -1290,8 +1354,8 @@ set_statusbar_msg (gchar * str)
 
   status_string = g_strdup (str);
 
-  gtk_statusbar_pop(statusbar, 0);
-  gtk_statusbar_push(statusbar, 0, status_string);
+  gtk_statusbar_pop(appdata.statusbar, 0);
+  gtk_statusbar_push(appdata.statusbar, 0, status_string);
 }				/* set_statusbar_msg */
 
 
@@ -1316,12 +1380,28 @@ canvas_link_delete(canvas_link_t *canvas_link)
 {
    /* Right now I'm not very sure in which cases there could be a canvas_link but not a link_item, but
    * I had a not in update_canvas_nodes that if the test is not done it can lead to corruption */
-  if (canvas_link->link_item)
+  if (canvas_link->src_item)
     {
-      gtk_object_destroy (GTK_OBJECT (canvas_link->link_item));
-      g_object_unref (G_OBJECT (canvas_link->link_item));
-      canvas_link->link_item = NULL;
+      gtk_object_destroy (GTK_OBJECT (canvas_link->src_item));
+      g_object_unref (G_OBJECT (canvas_link->src_item));
+      canvas_link->src_item = NULL;
+    }
+  if (canvas_link->dst_item)
+    {
+      gtk_object_destroy (GTK_OBJECT (canvas_link->dst_item));
+      g_object_unref (G_OBJECT (canvas_link->dst_item));
+      canvas_link->dst_item = NULL;
     }
 
   g_free (canvas_link);
 }
+
+void timeout_changed(void)
+{
+  /* When removing the source (which could either be an idle or a timeout
+   * function, I'm also forcing the callback for the corresponding 
+   * destroying function, which in turn will install a timeout or idle
+   * function using the new refresh_period. It might take a while for it
+   * to settle down, but I think it works now */
+  g_source_remove (diagram_timeout);
+}
