@@ -26,7 +26,7 @@
 #ifndef __UTIL_H__
 #define __UTIL_H__
 
-#include "globals.h"
+#include "appdata.h"
 
 #ifdef __cplusplus
 extern "C"
@@ -52,6 +52,17 @@ extern "C"
   const gchar *ipv6_to_str(const guint8 *ad);
   const gchar *address_to_str(const address_t * ad);
   const gchar *type_to_str(const address_t * ad);
+
+  /* xml helpers */
+  gchar *xmltag(const gchar *name, const gchar *fmt, ...);
+
+#if !defined(HAVE_G_QUEUE_INIT)
+  void compat_g_queue_init(GQueue *gq);
+  gchar *compat_gdk_color_to_string(const GdkColor *color);
+#define g_queue_init compat_g_queue_init
+#define gdk_color_to_string compat_gdk_color_to_string
+#endif
+
   
 #ifdef __cplusplus
 }
